<?php
  /**
   * Třída zpracovává uživatelský vstup týkající se správy lokálních správců, na základě kterého třída
   * volá odpovídající metody, přičemž svůj výstup předává další vrstvě pro výpis.
   *
   * @author Martin Šebela
   */
  class DeptContactsController extends Controller {
    /**
     * Zpracuje vstup z URL adresy a na základě toho zavolá odpovídající metodu.
     *
     * @param array $arguments         Uživatelský vstup
     */
    public function process($arguments) {
      $this->setView('header-departments-contacts', true);
      $this->setUrlSection('departments-contacts');

      $model = new DeptContactsModel();
      $formData = [
        'inputsNames' => ['email', 'department-building', 'department-name', 'note'],
        'formPrefix' => 'contact-',
        'dbTable' => 'blck_departmens_contacts'
      ];

      if (isset($_GET['action'])) {
        $id = isset($_GET['id']) ? get_int_from_string($_GET['id']) : false;

        if ($_GET['action'] == ACT_NEW) {
          $this->processNew($model, $formData);
        }
        elseif ($_GET['action'] == ACT_EDIT && $id !== false) {
          $this->processEdit($model, $formData, $id);
        }
        elseif ($_GET['action'] == ACT_DEL && $id !== false) {
          $this->processDelete($model, $id);
        }
        else {
          $this->addMessage(MSG_ERROR, 'Zvolená akce neexistuje.');
          $this->redirect($this->urlSection);
        }
      }
      else {
        $this->processList($model);
      }

      // Odkaz na nápovědu.
      $this->setHelpLink('https://support.zcu.cz/index.php/LPS:Seznam_lok%C3%A1ln%C3%ADch_spr%C3%A1vc%C5%AF');
    }


    /**
     * Vypíše formulář a obslouží s ním související prvky pro přidání nového lokálního správce.
     *
     * @param DeptContactsModel $model      Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     */
    private function processNew($model, $formData) {
      $this->setTitle('Nový lokální správce');
      $this->setView('form-department-contact');

      $model->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);
      $this->initViewData($model, ACT_NEW, $formData['formPrefix']);

      if (isset($_POST[$model->formPrefix . $this->getData('action')])) {
        try {
          $model->load($_POST);

          $model->validateData();
          $model->insertDeptContact();

          $this->addMessage(MSG_SUCCESS, 'Přidání proběhlo úspěšně.');
          $this->redirect($this->urlSection);
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }
      }
    }


    /**
     * Vypíše formulář a obslouží s ním související prvky pro úpravu konkrétního lokálního správce.
     *
     * @param DeptContactsModel $model      Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     * @param int $idContact                ID lokálního správce
     */
    private function processEdit($model, $formData, $idContact) {
      $this->setTitle('Úprava lokálního správce');
      $this->setView('form-department-contact');

      $model->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);
      $this->setViewData('contact', $model->getDeptContact($idContact));

      // Ověření existence záznamu.
      $this->checkRecordExistence($this->getData('contact'));

      $this->initViewData($model, ACT_EDIT, $formData['formPrefix']);

      if (isset($_POST[$model->formPrefix . $this->getData('action')])) {
        try {
          $model->load($_POST);

          $model->validateData();
          $model->updateDeptContact($idContact);

          $this->addMessage(MSG_SUCCESS, 'Úprava proběhla úspěšně.');
          $this->redirect($this->urlSection);
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }
      }
    }


    /**
     * Zavolá metodu pro odstranění konkrétního lokálního správce.
     *
     * @param DeptContactsModel $model     Instance třídy
     * @param int $idContact               ID lokálního správce
     */
    private function processDelete($model, $idContact) {
      if (isset($_POST)) {
        try {
          $model->isValidCsrfToken($_POST);
          $model->deleteDeptContact($idContact);

          $this->addMessage(MSG_SUCCESS, 'Smazání proběhlo úspěšně.');
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }

        $this->redirect($this->urlSection);
      }
    }


    /**
     * Vypíše seznam lokálních správců.
     *
     * @param DeptContactsModel $model      Instance třídy
     */
    private function processList($model) {
      $this->setTitle('Lokální správci');
      $this->setView('list-departments-contacts');

      $this->setViewData('departmentsContacts', $model->getDeptContacts());
    }
  }
