<?php
  /**
   * Třída zpracovává uživatelský vstup týkající se přidávání nových bezpečnostních incidentů,
   * na základě kterého volá odpovídající metody, přičemž svůj výstup předává další vrstvě pro výpis.
   *
   * @author Martin Šebela
   */
  class IncidentController extends Controller {
    /**
     * @var int         Aktuálně zobrazený krok ve formuláři.
     */
    private $currentStepInForm = 1;


    /**
     * Zpracuje vstup z URL adresy a na základě toho zavolá odpovídající metodu.
     *
     * @param array $arguments         Uživatelský vstup
     */
    public function process($arguments) {
      $this->setView('header-incident', true);
      $this->setUrlSection('incident');

      $model = new IncidentModel();
      $formData = [
        'inputsNames' => ['id-parent-ticket', 'id-child-ticket', 'parent-ticket-lang', 'id-type', 'blocked-user', 'blocked-ip', 'blocked-mac', 'user', 'user-lang', 'incident-datetime', 'incident-evidence', 'blocking-recipient', 'blocking-subject', 'blocking-message', 'notify-recipient', 'notify-message', 'unblocking-message'],
        'formPrefix' => 'incident-',
        'dbTable' => 'blck_incidents'
      ];

      $steps = $model->getIncidentStatuses();

      // Aktuální krok ve formuláři vyžadovaný uživatelem.
      if (isset($_GET['step']) && is_numeric($_GET['step']) && $_GET['step'] > 0 && $_GET['step'] <= count($steps)) {
        $this->currentStepInForm = $_GET['step'];
      }

      $this->setViewData('steps', $steps);

      if (isset($_GET['action']) && !empty($_GET['action'])) {
        if ($_GET['action'] == ACT_RESET) {
          $model->reset(ACT_NEW);
          $this->redirect($this->urlSection);
        }
        else {
          $this->addMessage(MSG_ERROR, 'Zvolená akce neexistuje.');
          $this->redirect($this->urlSection);
        }
      }
      else {
        $this->processNew($model, $formData);
      }

      // Odkaz na nápovědu.
      $this->setHelpLink('');
    }


    /**
     * Vypíše formulář a obslouží s ním související prvky pro přidání nového bezpečnostního incidentu.
     *
     * @param IncidentModel $model          Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     */
    private function processNew($model, $formData) {
      $this->setTitle('Nový bezpečnostní incident');
      $this->setView('form-incident');

      $this->setViewData('currentStep', $this->currentStepInForm);

      $incident = $model->getTempIncident(ACT_NEW);
      $this->setViewData('incident', $incident);
      $lastStep = (isset($incident['last_step'])) ? $incident['last_step'] : 1;

      // Pokud již došlo k založení incidentu (má své ID v databázi), pak uživatele přesměrovat do editace incidentu.
      if (!empty($incident['id_incident'])) {
        $model->reset(ACT_NEW);
        $this->redirect('list-incidents/' . ACT_EDIT . '/' . $incident['id_incident'] . '?step=' . $lastStep);
      }

      $this->setViewData('incidentLastStep', $lastStep);

      // Pokud uživatel úmyslně přeskočí na budoucí krok přes URL, přesměrovat jej na poslední, úspěšně dokončený krok.
      if ($this->currentStepInForm > $lastStep) {
        $this->redirect($this->urlSection . '?step=' . $lastStep);
      }

      // Načtení možných jazykových mutací z databáze u druhého kroku.
      if ($this->currentStepInForm == 2) {
        $this->setViewData('incidentLanguages', EmailTemplatesModel::getEmailTemplatesLanguages($incident['id_type'], false));
      }

      $model->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);
      $model->setCurrentStep($this->currentStepInForm);
      $this->initViewData($model, ACT_NEW, $formData['formPrefix']);

      // Načtění dat z databáze pro vstupní pole.
      $this->setViewData('types', IncidentTypeModel::getIncidentTypes());

      if (isset($_POST[$model->formPrefix . $this->getData('action')])) {
        try {
          $model->load($_POST);

          $model->validateData();
          $nextStep = $model->makeIncidentStep($this->currentStepInForm, ACT_NEW);

          // Při úspěšném přidání incidentu do databáze vyresetovat formulář
          // a uživatele přesměrovat na stránku se seznamem incidentů.
          if ($nextStep >= INCIDENT_FINAL_STEP_ID) {
            $model->reset(ACT_NEW);

            $this->addMessage(MSG_SUCCESS, 'Incident byl úspěšně vyřešen.');
            $this->redirect('list-incidents');
          }

          $this->redirect($this->urlSection . '?step=' . $nextStep);
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }
      }
    }
  }
