<?php
  /**
   * Třída zpracovává uživatelský vstup týkající se správy bezpečnostních incidentů, na základě
   * kterého volá odpovídající metody, přičemž svůj výstup předává další vrstvě pro výpis.
   *
   * @author Martin Šebela
   */
  class ListIncidentsController extends Controller {
    /**
     * @var int         Aktuálně zobrazený krok ve formuláři.
     */
    private $currentStepInForm = 1;


    /**
     * Zpracuje vstup z URL adresy a na základě toho zavolá odpovídající metodu.
     *
     * @param array $arguments         Uživatelský vstup
     */
    public function process($arguments) {
      $this->setView('header-list-incidents', true);
      $this->setUrlSection('list-incidents');

      $model = new IncidentModel();
      $formData = [
        'inputsNames' => ['step', 'id-parent-ticket', 'parent-ticket-lang', 'id-type', 'blocked-user', 'blocked-ip', 'blocked-mac', 'user', 'user-lang', 'incident-datetime', 'incident-evidence', 'blocking-recipient', 'blocking-subject', 'blocking-message', 'notify-recipient', 'notify-message', 'unblocking-message'],
        'formPrefix' => 'incident-',
        'dbTable' => 'blck_incidents'
      ];

      if (isset($_GET['action'])) {
        $id = isset($_GET['id']) ? get_int_from_string($_GET['id']) : false;

        if ($_GET['action'] == ACT_EDIT && $id !== false) {
          $this->processEdit($model, $formData, $id);
        }
        else {
          $this->addMessage(MSG_ERROR, 'Zvolená akce neexistuje.');
          $this->redirect($this->urlSection);
        }
      }
      else {
        $this->processList($model);
      }

      // Odkaz na nápovědu.
      $this->setHelpLink('');
    }


    /**
     * Vypíše formulář a obslouží s ním související prvky pro úpravu konkrétního bezpečnostního incidentu.
     *
     * @param IncidentModel $model          Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     * @param int $idIncident               ID bezpečnostního incidentu
     */
    private function processEdit($model, $formData, $idIncident) {
      $this->setTitle('Úprava bezpečnostního incidentu');
      $this->setView('form-incident');

      $model->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);

      $incident = $model->getIncident($idIncident);
      $this->setViewData('incident', $incident);
      $this->setViewData('incidentLastStep', $incident['id_status']);
      $this->setViewData('countSuccessSteps', $incident['id_status']);

      // Ověření existence záznamu.
      $this->checkRecordExistence($incident['id_incident']);

      $steps = $model->getIncidentStatuses($incident['id_incident']);

      // Aktuální krok ve formuláři vyžadovaný uživatelem.
      if (isset($_GET['step']) && is_numeric($_GET['step']) && $_GET['step'] > 0 && $_GET['step'] < count($steps)) {
        $this->currentStepInForm = $_GET['step'];
      }

      $this->setViewData('currentStep', $this->currentStepInForm);
      $this->setViewData('steps', $steps);

      // Pokud uživatel úmyslně přeskočí na budoucí krok přes URL, přesměrovat jej na poslední, úspěšně dokončený krok.
      if ($this->currentStepInForm > $incident['id_status']) {
        $this->redirect($this->urlSection . '/edit/' . $idIncident . '?step=' . $incident['id_status']);
      }

      // Pokud je požadován krok "Upozornění stěžovatele", ale incident tomu neodpovídá...
      if ($this->currentStepInForm == 4 && empty($incident['id_parent_ticket'])) {
        $this->redirect($this->urlSection . '/edit/' . $idIncident . '?step=' . $incident['id_status']);
      }

      $model->setCurrentStep($this->currentStepInForm);

      // Načtení možných jazykových mutací z databáze u druhého kroku.
      if ($this->currentStepInForm == 2) {
        $this->setViewData('incidentLanguages', EmailTemplatesModel::getEmailTemplatesLanguages($incident['id_type'], false));
      }

      $this->initViewData($model, ACT_EDIT, $formData['formPrefix']);

      // Data z databáze pro vstupní pole.
      $this->setViewData('types', IncidentTypeModel::getIncidentTypes());

      if (isset($_POST[$model->formPrefix . $this->getData('action')])) {
        try {
          $model->load($_POST);

          $model->validateData();
          $nextStep = $model->makeIncidentStep($this->currentStepInForm, ACT_EDIT, $idIncident);

          if ($nextStep >= INCIDENT_FINAL_STEP_ID) {
            $model->reset(ACT_EDIT);

            $this->addMessage(MSG_SUCCESS, 'Incident byl úspěšně vyřešen.');
            $this->redirect($this->urlSection);
          }

          $this->redirect($this->urlSection . '/edit/' . $idIncident . '?step=' . $nextStep);
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }
      }
    }


    /**
     * Vypíše seznam všech řešených bezpečnostních incidentů.
     *
     * @param IncidentModel $model          Instance třídy
     */
    private function processList($model) {
      $this->setTitle('Seznam incidentů');
      $this->setView('list-incidents');

      $this->setViewData('incidents', $model->getIncidents());
    }
  }
