<?php
  /**
   * První třída, která zpracovává uživatelský vstup z URL adresy, na základě něhož zavolá odpovídající metodu
   * nebo vytvoří odpovídající Controller, kterému uživatelské argumenty dál předá.
   *
   * @author Martin Šebela
   */
  class RouterController extends Controller {
    /**
     * @var mixed       Vybraný Controller, který bude zpracovávat uživatelské požadavky.
     */
    protected $controller;

    /**
     * @var array       Seznam všech dostupných Controllerů.
     */
    private $existsControllers;


    /**
     * Konstruktor nastavující výchozí hodnoty instance třídy (resp. seznam všech dostupných Controllerů).
     */
    public function __construct() {
      $this->controller = null;

      $this->existsControllers = [
        'incident' => 'IncidentController',
        'list-incidents' => 'ListIncidentsController',
        'types-templates' => 'IncidentTypesController',
        'departments-contacts' => 'DeptContactsController'
      ];
    }


    /**
     * Zpracuje vstup z URL adresy a na základě toho zavolá odpovídající metodu.
     *
     * @param string $arguments        Uživatelský vstup
     */
    public function process($arguments) {
      // Uživatel je přihlášen a je v autorizované části aplikace.
      if (PermissionsModel::getUserId() != null) {
        if (isset($_GET['section'])) {
          $this->controller = $this->getController($_GET['section']);
        }
        elseif (isset($_GET['logout'])) {
          $this->logout();
        }
        else {
          $this->controller = new DashboardController();
        }
      }

      // Aktivovat Controller určený pro danou stránku (resp. sekci).
      $this->setControllerData($arguments);
    }


    /**
     * Vrátí odpovídající Controller v závislosti na předaném argumentu.
     *
     * @param string $controllerName   Název Controlleru
     * @param bool $public             TRUE pokud se jedná o Controller přístupný na veřejné části aplikace, jinak FALSE
     * @return mixed|null              Instance Controlleru nebo NULL při nenalezení odpovídajícího Controlleru
     */
    private function getController($controllerName, $public = false) {
      if (empty($controllerName)) {
        return null;
      }

      if (array_key_exists($controllerName, $this->existsControllers)) {
        return new $this->existsControllers[$controllerName]();
      }

      if ($public == false) {
        // Chybová hláška o nenalezení sekce v interní části aplikace.
        return new ErrorNotFoundController();
      }
      else {
        $this->redirect(null, 1);
      }
    }


    /**
     * Předá uživatelský vstup a data instanci Controlleru.
     *
     * @param string $arguments        Uživatelský vstup
     */
    private function setControllerData($arguments) {
      if ($this->controller != null) {
        $this->controller->process($arguments);

        $this->setViewData('html_title', $this->controller->getTitle());

        $this->setViewData('menu', $this->getMenu());
        $this->setViewData('messages', $this->getMessages());

        $this->setView('base');
      }
    }


    /**
     * Odhlásí uživatele ze systému a přesměruje ho do kořene webu.
     */
    private function logout() {
      PermissionsModel::logout();

      $this->redirect(null, 1);
    }
  }
