<?php
  /**
   * Třída sloužící k získávání informací o lokálních správcích, k přidávání nových lokálních správců,
   * k úpravě informací o těch již existujících a k dalším souvisejícím operacím.
   *
   * @author Martin Šebela
   */
  class DeptContactsModel extends FormModel {
    /**
     * @var string      Název tabulky v databázi, se kterou je instance svázána.
     */
    private $dbTableName = 'blck_departmens_contacts';

    /**
     * @var string      Jméno a příjmení lokálního správce (získáno jako údaj z LDAP).
     */
    protected $fullname;

    /**
     * @var string      Kontaktní e-mail na lokálního správce.
     */
    protected $email;

    /**
     * @var string      Budova (např. zkratka fakulty), ve které sídlí oddělení, kterou má lokální správce na starost.
     */
    protected $departmentBuilding;

    /**
     * @var string      Název oddělení (zkratka), kterou má lokální správce na starost.
     */
    protected $departmentName;

    /**
     * @var string      Poznámka k lokálnímu správci.
     */
    protected $note;


    /**
     * Připraví nového lokálního správce v závislosti na vyplněných datech a vrátí ho formou pole.
     *
     * @return array                   Pole obsahující data o lokálním správci.
     */
    public function makeDeptContact() {
      return [
        'email' => $this->email,
        'department_building' => $this->departmentBuilding,
        'department_name' => $this->departmentName,
        'note' => $this->note
      ];
    }


    /**
     * Uloží do instance a zároveň vrátí (z databáze) informace o zvoleném lokálním správci.
     *
     * @param int $id                  ID lokálního správce
     * @return array                   Pole obsahující informace o lokálním správci.
     */
    public function getDeptContact($id) {
      $this->dbRecordData = Database::querySingle('
              SELECT `id_department_contact`, `email`, `department_building`, `department_name`, `note`
              FROM `blck_departmens_contacts`
              WHERE `id_department_contact` = ?
              AND `visible` = 1'
      , $id);

      if (!empty($this->dbRecordData['email'])) {
        $ldap = new LdapModel();

        // Načtení jména a příjmení z LDAP.
        $this->dbRecordData['fullname'] = $ldap->getUserCNByUsername(
          get_email_part($this->dbRecordData['email'], 'username')
        );
      }

      return $this->dbRecordData;
    }


    /**
     * Vrátí seznam všech lokálních správců z databáze.
     *
     * @return mixed                   Pole lokálních správců a informace o každém z nich.
     */
    public static function getDeptContacts() {
      $data = Database::queryMulti('
              SELECT `id_department_contact`, `id_by_user`, `email`, `department_building`, `department_name`, `note`,
              `username`,
              DATE_FORMAT(blck_departmens_contacts.date_added, "%e. %c. %Y") AS date_added
              FROM `blck_departmens_contacts`
              JOIN `blck_users`
              ON blck_departmens_contacts.id_by_user = blck_users.id_user
              WHERE blck_departmens_contacts.visible = 1
              ORDER BY `department_building`, `department_name`
      ');

      $ldap = new LdapModel();

      foreach ($data as &$record) {
        // Načtení jména a příjmení z LDAP.
        $record['fullname'] = $ldap->getUserCNByUsername(get_email_part($record['email'], 'username'));
      }

      return $data;
    }


    /**
     * Vrátí aktuální počet aktivních lokálních správců z databáze.
     *
     * @return int                     Celkový počet aktivních lokálních správců
     */
    public static function getDeptContactsCount() {
      return Database::queryCount('
              SELECT COUNT(*)
              FROM `blck_departmens_contacts`
              WHERE `visible` = 1');
    }


    /**
     * Vloží do databáze nového lokálního správce.
     */
    public function insertDeptContact() {
      $contact = $this->makeDeptContact();

      $contact['id_by_user'] = $_SESSION['user']['id'];
      $contact['date_added'] = date('Y-m-d H:i:s');

      Logger::info('Vložení nového lokálního správce.', $contact);

      Database::insert($this->getDbTableName(), $contact);
    }


    /**
     * Upraví zvoleného lokálního správce.
     *
     * @param int $id                  ID lokálního správce
     */
    public function updateDeptContact($id) {
      $contact = $this->makeDeptContact();

      Logger::info('Úprava existujícího lokálního správce.', $contact);

      Database::update(
        $this->getDbTableName(),
        $contact,
        'WHERE `id_department_contact` = ? AND `visible` = 1',
        $id
      );
    }


    /**
     * Odstraní (resp. deaktivuje) lokálního správce z databáze.
     *
     * @param int $id                  ID lokálního správce
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function deleteDeptContact($id) {
      $result = Database::update(
        'blck_departmens_contacts',
        ['visible' => 0],
        'WHERE `id_department_contact` = ? AND `visible` = 1',
        $id
      );

      if ($result == 0) {
        Logger::warning('Snaha o smazání neexistujícího lokálního správce.', $id);

        throw new UserError('Záznam vybraný ke smazání neexistuje.', MSG_ERROR);
      }

      Logger::info('Smazání existujícího lokálního správce.', $id);
    }


    /**
     * Zkontroluje uživatelský vstup (atributy třídy), který se bude zapisovat do databáze.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function validateData() {
      $this->isEmailEmpty();
      $this->isEmailTooLong();
      $this->isEmailValid();
      $this->isEmailExists();

      $this->isDepartmentBuildingEmpty();
      $this->isDepartmentBuildingTooLong();

      $this->isDepartmentNameTooLong();

      $this->isNoteTooLong();
    }


    /**
     * Ověří, zdali byl vyplněn e-mail lokálního správce.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isEmailEmpty() {
      if (empty($this->email)) {
        throw new UserError('Není vyplněn e-mail lokálního správce.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaný e-mail lokálního správce není příliš dlouhý.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isEmailTooLong() {
      if (mb_strlen($this->email) > $this->inputsMaxLengths['email']) {
        throw new UserError('E-mail lokálního správce je příliš dlouhý.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali je zadaný e-mail lokálního správce validní.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isEmailValid() {
      if (!filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
        throw new UserError('E-mail lokálního správce je v nesprávném formátu.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaný e-mail lokálního správce existuje v LDAP.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isEmailExists() {
      $ldap = new LdapModel();

      if ($ldap->getEmailByUsername(get_email_part($this->email, 'username')) == null) {
        throw new UserError('Zadaný e-mail lokálního správce neexistuje.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali byla vyplněna zkratka fakulty (nebo budovy), kterou má lokální správce na starost.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isDepartmentBuildingEmpty() {
      if (empty($this->departmentBuilding)) {
        throw new UserError('Není vyplněna zkratka fakulty, kterou má lokální správce na starost.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaná zkratka fakulty (nebo budovy) není příliš dlouhá.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isDepartmentBuildingTooLong() {
      if (mb_strlen($this->departmentBuilding) > $this->inputsMaxLengths['department-building']) {
        throw new UserError('Zkratka fakulty je příliš dlouhá.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaná zkratka katedry fakulty (nebo zkratka oddělení) není příliš dlouhá.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isDepartmentNameTooLong() {
      if (mb_strlen($this->departmentName) > $this->inputsMaxLengths['department-name']) {
        throw new UserError('Zkratka katedry je příliš dlouhá.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali není zadaná poznámka o lokálním správci příliš dlouhá.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isNoteTooLong() {
      if (mb_strlen($this->note) > $this->inputsMaxLengths['note']) {
        throw new UserError('Poznámka o lokálním správci je příliš dlouhá.', MSG_ERROR);
      }
    }
  }
