<?php
  /**
   * Třída určená pro komunikaci a vyhledávání v LDAP.
   *
   * @author Martin Šebela
   */
  class LdapModel {
    /**
     * @var resource    LDAP připojení nebo NULL.
     */
    private $ldapConnection;


    /**
     * Konstruktor třídy automaticky volající metodu pro připojení k LDAP.
     */
    public function __construct() {
      $this->connect();
    }


    /**
     * Připojí se k LDAP a nastaví verzi používaného protokolu.
     */
    private function connect() {
      $this->ldapConnection = ldap_connect(LDAP_HOSTNAME, LDAP_PORT);

      ldap_set_option($this->ldapConnection, LDAP_OPT_PROTOCOL_VERSION, 3);
    }


    /**
     * Odpojí se z LDAP.
     */
    public function close() {
      ldap_unbind($this->ldapConnection);
    }


    /**
     * Vyhledá a vrátí data z LDAP na základě zadaných parametrů.
     *
     * @param string $ldapDn           Cesta v LDAP, ve které se bude vyhledávat.
     * @param string $filter           Filtr, na základě kterého se bude vyhledávat.
     * @return array|null              Vyhledaná data nebo NULL
     */
    private function getDataByFilter($ldapDn, $filter) {
      if ($this->ldapConnection && !empty($ldapDn) && !empty($filter)) {
        $ldapbind = ldap_bind($this->ldapConnection);

        if ($ldapbind) {
          $sr = ldap_search($this->ldapConnection, $ldapDn . ',' . LDAP_BASE_DN, $filter);
          $countRecords = ldap_count_entries($this->ldapConnection, $sr);

          if ($countRecords == 1) {
            return ldap_get_entries($this->ldapConnection, $sr);
          }
        }
      }
      else {
        Logger::error(
          'Nepodařilo se získat data z LDAP.',
          ['ldap_dn' => $ldapDn, 'filter' => $filter]
        );
      }

      return null;
    }


    /**
     * Vrátí e-mail uživatele (první v pořadí) v závislosti na jeho uživatelském jménu z LDAP.
     *
     * @param string $username         Uživatelské jméno
     * @return string|null             E-mail uživatele nebo NULL
     */
    public function getEmailByUsername($username) {
      if (!empty($username)) {
        $info = $this->getDataByFilter('ou=Users', 'uid=' . $username);

        if (isset($info['count'])) {
          for ($i = 0; $i < $info['count']; $i++) {
            return $info[$i]['mail'][0];
          }
        }
      }

      return null;
    }


    /**
     * Vrátí jméno a příjmení uživatele v závislosti na jeho uživatelském jménu z LDAP.
     *
     * @param string $username         Uživatelské jméno
     * @return string|null             Jméno a příjmení uživatele nebo NULL
     */
    public function getUserCNByUsername($username) {
      if (!empty($username)) {
        $info = $this->getDataByFilter('ou=Users', 'uid=' . $username);

        if (isset($info['count'])) {
          for ($i = 0; $i < $info['count']; $i++) {
            $name = '';

            if (isset($info[$i]['givenname;lang-cs'][0])) {
              $name .= $info[$i]['givenname;lang-cs'][0];
            }

            if (isset($info[$i]['sn;lang-cs'][0])) {
              $name .= ' ' . $info[$i]['sn;lang-cs'][0];
            }

            return $name;
          }
        }
      }

      return null;
    }
  }
