<?php
  /**
   * Třída sloužící k uchování informací o identitě, jež se má zablokovat nebo odblokovat.
   * Instance třídy očekávají jednotlivé moduly - Blockátory.
   *
   * @author Martin Šebela
   */
  class BlockedIdentity {
    /**
     * @var string      IP adresa zařízení k (od)blokování.
     */
    private $ipAddress;

    /**
     * @var string      MAC adresa zařízení k (od)blokování.
     */
    private $macAddress;

    /**
     * @var string      Identita (e-mail) k (od)blokování.
     */
    private $identity;

    /**
     * @var string      Uživatelské jméno (tzn. bez domény organizace) k (od)blokování.
     */
    private $username;

    /**
     * @var int         Číslo (identifikátor) lístku v RT systému, ve kterém je bezpečnostní incident veden.
     */
    private $idRequestTracker;

    /**
     * @var int         Identifikátor bezpečnostního incidentu v systému Blockator.
     */
    private $idIncident;

    /**
     * @var string      Textový důvod blokace (např. typ bezpečnostního incidentu).
     */
    private $reason;


    /**
     * Vytvoří identitu, která se má zablokovat nebo odblokovat.
     *
     * @param string $ipAddress        IP adresa zařízení k (od)blokování.
     * @param string $macAddress       MAC adresa zařízení k (od)blokování.
     * @param string $identity         Identita (e-mail) k (od)blokování.
     * @param int $idRequestTracker    Číslo (identifikátor) lístku v RT systému.
     * @param int $idIncident          Identifikátor bezpečnostního incidentu v systému Blockator.
     * @param string $reason           Důvod provedeného úkonu.
     */
    public function __construct($ipAddress, $macAddress, $identity, $idRequestTracker, $idIncident, $reason) {
      $this->ipAddress = $ipAddress;
      $this->macAddress = $macAddress;
      $this->identity = $identity;
      $this->idRequestTracker = $idRequestTracker;
      $this->idIncident = $idIncident;
      $this->reason = $reason;

      $username = get_email_part($identity, 'username');
      $this->username = ($username != null) ? $username : $this->identity;
    }


    /**
     * Vrátí IP adresu zařízení k (od)blokování.
     *
     * @return string                  IP adresa.
     */
    public function getIpAddress() {
      return $this->ipAddress;
    }


    /**
     * Vrátí MAC adresu zařízení k (od)blokování.
     *
     * @return string                  MAC adresa zařízení.
     */
    public function getMacAddress() {
      return $this->macAddress;
    }


    /**
     * Vrátí identitu (e-mail) k (od)blokování.
     *
     * @return string                  Identita (e-mail).
     */
    public function getIdentity() {
      return $this->identity;
    }


    /**
     * Vrátí uživatelské jméno k (od)blokování.
     *
     * @return string                  Uživatelské jméno.
     */
    public function getUsername() {
      return $this->username;
    }


    /**
     * Vrátí číslo (identifikátor) lístku v RT systému, ve kterém je bezpečnostní incident veden.
     *
     * @return int                     Identifikátor lístku v RT systému, ve kterém je bezpečnostní incident veden.
     */
    public function getIdRequestTracker() {
      return $this->idRequestTracker;
    }


    /**
     * Vrátí identifikátor bezpečnostního incidentu v systému Blockator.
     *
     * @return int                     Identifikátor bezpečnostního incidentu v systému Blockator.
     */
    public function getIdIncident() {
      return $this->idIncident;
    }


    /**
     * Vrátí důvod provedného úkonu (blokace, odblokování).
     *
     * @return string                  Důvod blokace.
     */
    public function getReason() {
      return $this->reason;
    }
  }